# Generated by Django 3.1.14 on 2022-10-12 14:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_highlighter', '0005_remove_annotation_entity_link'),
    ]

    operations = [
        migrations.AlterField(
            model_name='activelearningproject',
            name='sampling_strategy',
            field=models.CharField(choices=[], max_length=4),
        ),
    ]
